from django.contrib.auth.decorators import user_passes_test
from userroles.models import UserRole
from userroles import roles


def role_required(*user_roles):
    """
    Decorator for views that checks whether a user has a particular role,
    redirecting to the log-in page if neccesary.
    Follows same style as django.contrib.auth.decorators.login_required,
    and django.contrib.auth.decorators.permission_required.
    """    
    required_roles = []
    for each in user_roles:
        if isinstance(each, basestring):
            required_roles.append( roles.get(each) )
        else:
            required_roles.append(each)
    
    def check_role(user):
        try:
            user_role = getattr(user, 'role', None)
            if not user_role:
                return False
            for each in required_roles:
                if user_role == each or user_role.subrole_of( each ):
                    return True
            return False
        except UserRole.DoesNotExist:
            return False
    return user_passes_test(check_role)



from django.conf import settings
from django.db import models
from userroles import roles


class UserRole(models.Model):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, related_name='role')
    name = models.CharField(max_length=100, choices=roles.choices)
    child = models.CharField(max_length=100, blank=True)
    _valid_roles = roles

    @property
    def profile(self):
        if not self.child:
            return None
        return getattr(self, self.child)

    def __eq__(self, other):
        return self.name == other.name

    def __getattr__(self, name):
        if name.startswith('is_'):
            role = getattr(self._valid_roles, name[3:], None)
            if role:
                return self == role or self.subrole_of( role )
        role = roles.get(self.name)
        if hasattr(role, name):
            return getattr(role, name)
        raise AttributeError("'%s' object has no attribute '%s'" %
                              (self.__class__.__name__, name))

    def __unicode__(self):
        return self.name


def set_user_role(user, role):
    role_name = role if isinstance(role, basestring) else role.name
    try:
        profile = UserRole.objects.get(user=user)
    except UserRole.DoesNotExist:
        profile = UserRole(user=user, name=role_name)
    else:
        profile.name = role_name
    profile.save()




from django.test import TestCase
from django.conf import settings
from django.contrib.auth import get_user_model
from django.test.client import Client
from milkman.dairy import milkman
from userroles.models import set_user_role, UserRole
from userroles.utils import SettingsTestCase
from userroles import roles, Roles, Role

User = get_user_model()


class UserRoleTests(TestCase):
    """
    Test basic user.role operations.
    """

    def setUp(self):
        super(UserRoleTests, self).setUp()
        self.user = self.create_user()
        set_user_role(self.user, roles.manager)

    def create_user(self):
        return milkman.deliver(User)

    def test_role_comparison(self):
        """
        Ensure that we can test if a user role has a given value.
        """
        self.assertEquals(self.user.role, roles.manager)

    def test_role_in_set(self):
        """
        Ensure that we can test if a user role exists in a given set.
        """
        self.assertIn(self.user.role, (roles.manager,))

    def test_is_role(self):
        """
        Test `user.role.is_something` style.
        """
        self.assertTrue(self.user.role.is_manager)

    def test_is_not_role(self):
        """
        Test `user.role.is_not_something` style.
        """
        self.assertFalse(self.user.role.is_moderator)

    def test_is_invalid_role(self):
        """
        Test `user.role.is_invalid` raises an AttributeError.
        """
        self.assertRaises(AttributeError, getattr, self.user.role, 'is_foobar')
    
    def test_choices(self):
        choices = [each[0] for each in roles.choices]
        for role in getattr( settings, 'USER_ROLES' ):
            for subrole in getattr( settings, role.upper()+'_ROLES', () ):
                self.assertTrue( subrole in choices )
            self.assertTrue( role in choices )
    
    def test_get_roles(self):
        for each in getattr(settings, 'USER_ROLES', ()):
            self.assertTrue( roles.get(each) == getattr(roles, each) )
            self.assertTrue( isinstance(roles.get(each), Role) )
            
    def test_set_role(self):
        role_name = getattr( settings, 'USER_ROLES' )[0]
        user = self.create_user()
        set_user_role(user, role_name)
        self.assertTrue( user.role == roles.get(role_name) )
        self.assertTrue( getattr(user.role, 'is_'+role_name) )
        for each in getattr( settings, 'USER_ROLES' )[1:]:
            self.assertFalse( user.role == roles.get(each) )
            self.assertFalse( getattr(user.role, 'is_'+each) )
    
    def test_set_subrole(self):
        for role_name in getattr( settings, 'USER_ROLES' ):
            for subrole_name in getattr( settings, role_name.upper()+'_ROLES', () ):
                user = self.create_user()
                set_user_role(user, subrole_name)
                self.assertTrue( user.role == roles.get(subrole_name) )
                self.assertTrue( getattr(user.role, 'is_'+subrole_name) )
                self.assertTrue( getattr(user.role, 'is_'+role_name) )
            user = self.create_user()
            set_user_role(user, role_name)
            self.assertTrue( user.role == roles.get(role_name) )
            self.assertTrue( getattr(user.role, 'is_'+role_name) )
            self.assertFalse( getattr(user.role, 'is_'+subrole_name) )
        
    def test_subroles(self):
        for each in getattr( settings, 'USER_ROLES' ):
            role     = each
            subroles = getattr( settings, each.upper()+'_ROLES', () )
            for subrole in subroles:
                self.assertTrue( getattr(roles, subrole).subrole_of(role) )
                self.assertFalse( getattr(roles, subrole).subrole_of('some_user_role') )
                self.assertFalse( getattr(roles, role).subrole_of(subrole) )
                self.assertFalse( getattr(roles, subrole).has_subrole(role) )
                self.assertFalse( getattr(roles, subrole).has_subrole('some_user_role') )
                self.assertTrue( getattr(roles, role).has_subrole(subrole) )
            self.assertFalse( getattr(roles, role).subrole_of('some_user_role') )
            self.assertFalse( getattr(roles, role).has_subrole('some_user_role') )
            



from setuptools import setup, find_packages

setup(
    name             = 'custom-user-roles',
    version          = '1.1.0',
    # original author = 'Tom Christie'
    # original author_email  = 'tom@tomchristie.com'
    author           = "Diogo Laginha",
    author_email     = "diogo.laginha.machado@gmail.com",
    url              = 'https://github.com/laginha/django-user-roles',
    description      = "Simple role-based user permissions for Django.",
    packages         = find_packages(where='src'),
    package_dir      = {'': 'src'},
    install_requires = ['django', 'milkman'],
    extras_require   = {},
    zip_safe         = False,
    license          = 'MIT',
    classifiers      = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Intended Audience :: Developers',
    ]
)




from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class Role(object):
    """
    A single role, eg as returned by `roles.moderator`.
    """
    
    def add_subrole(self, role):
        self.subroles[ role.name ] = role
    
    def subrole_of(self, role):
        if isinstance(role, basestring):
            return bool(self.parent) and self.parent.name == role
        return bool(self.parent) and self.parent.name == role.name
    
    def has_subrole(self, role):
        if isinstance(role, basestring):
            return role in self.subroles
        return role.name in self.subroles

    def __init__(self, name, parent=None):
        self.name = name
        self.subroles = {}
        self.parent = parent

    def __unicode__(self):
        return self.name


class Roles(object):

    def get(self, userrole, *args):
        return getattr(self, userrole, *args)
    
    def add(self, name, parent=None):
        role = Role(name=name, parent=parent)
        setattr(self, name, role)
        self.choices.append( (name, name) )
        return role
    
    def __init__(self, config=None):
        """
        By default the Roles object will be created using configuration from
        the django settings file, but you can also set the configuration
        explicitly, for example, when testing.
        """
        self._config = config or getattr(settings, 'USER_ROLES', ())
        # a list of two-tuples of role names, suitable for use as the
        # 'choices' argument to a model field.
        self.choices = []
        
        def get_subroles(parent):
            subroles = getattr(settings, parent.name.upper()+'_ROLES', ())
            for each in subroles:
                role = self.add( each, parent=parent )
                parent.add_subrole( role )
                get_subroles( role )
        
        for item in self._config:
            if isinstance(item, basestring):
                # An item like 'manager'
                get_subroles( self.add( item ) )
            else:
                # Anything else
                raise ImproperlyConfigured(_INCORRECT_ARGS)


roles = Roles()
